import argparse
import logging
import sys
from typing import Optional

import colorama
from .models import HardwareLifecycle, SoftwareLifecycle
from .tools import Database, Downloader


def main() -> None:
    logging.basicConfig(filename='eol.log',
                        encoding='utf-8',
                        format='%(asctime)s %(message)s',
                        level=logging.DEBUG)

    excepthook = logging.error
    logging.debug('eol_checker started')

    colorama.init(autoreset=True)
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description="Query EOL software or hardware.")
    if len(sys.argv) < 2:
        parser.print_help()
    parser.add_argument("--software", dest="query_software",
                        type=str, required=False,
                        help="Query the software by name")
    parser.add_argument("--hardware", dest="query_hardware",
                        type=str,
                        required=False,
                        help="Query the software by name")
    parser.add_argument("-u", "--update",
                        dest="update_db",
                        action='store_true',
                        required=False,
                        help="Updates the local database. When combined with a query, it updates the database before running the query.")
    args: argparse.Namespace = parser.parse_args()
    logging.debug('eol_checker parameters: %s', args)

    download: Downloader = Downloader()

    logging.debug('Opening database connection')
    db: Database = Database('eol.db')

    softwareCount: int = 0
    hardwareCount: int = 0

    if (args.update_db is True):
        logging.debug('Starting download')
        newEolSoftware: list[SoftwareLifecycle] = download.get_eol_software(
        )
        newEolHardware: list[HardwareLifecycle] = download.get_eol_hardware(
        )
        logging.debug('Finished download')

        logging.debug('Updating database')
        success: bool = db.save(
            software_list=newEolSoftware, hardware_list=newEolHardware)

        if (success):
            logging.debug('Updated database')
            print("Updated the database.")

    if (args.query_software is not None):
        logging.debug('Querying for keyword: %s', args.query_software)

        eolSoftwareList: Optional[list[SoftwareLifecycle]] = db.search_software(
            args.query_software)

        if (eolSoftwareList is None):
            logging.debug("No software matches found with keyword.")
            print("No software matches found with keyword.")
        else:
            softwareCount: int = len(eolSoftwareList)
            for eolSoftware in eolSoftwareList:
                logging.debug("Software found: %s", eolSoftware)
                print(eolSoftware)
            print('Total ' + str(softwareCount) + ' software records found.')

    if (args.query_hardware is not None):
        logging.debug('Querying for keyword: %s', args.query_hardware)
        eolHardwareList: Optional[list[HardwareLifecycle]] = db.search_hardware(
            args.query_hardware)

        if (eolHardwareList is None):
            logging.debug("No hardware matches found with keyword.")
            print("No hardware matches found with keyword.")
        else:
            hardwareCount: int = len(eolHardwareList)
            for eolHardware in eolHardwareList:
                logging.debug("Hardware found: %s", eolHardware)
                print(eolHardware)
            print('Total ' + str(hardwareCount) + ' hardware records found.')

    db.close()
    logging.debug('Closed database connection')
    logging.debug('Exiting')


if __name__ == '__main__':
    main()
